
 /* Project: Open Source Gmini Tools file: Emulator.cpp

    Copyright (C) 2004 The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
    later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
    details.

    You should have received a copy of the GNU General Public License along with this program; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

#include "Emulator.h"
#include "util.h"
#include <fcntl.h>

#define DEFAULT_VER CPUCore::gm120v1100

Emulator::Emulator( int argc, const char *argv[] )
   :cpu( this ), memory( this ), io( this ), lcdi( this ),
    i2c( this ), lcd( this ), adc( this ), tracer( this ) {

   const char* hdFileName = "hda";

   symbols = new SymMap(  );

   for (int i = 0; i<argc; i++) {
      const char* fn = argv[i];
      const char* ext = fileextension(fn);

      if (strcmp(ext, ".aaz") == 0) {
         if( !memory.load( fn ) ) {
            console.output( "Failed to load firmware file: %s.\n", fn );
            abort(  );
         }
      } else if (strcmp(ext, ".map") == 0) {
         symbols->load( fn );
      } else if (strncmp(fn, "-hd=", 4) == 0) {
         hdFileName = fn + 4;
      } else {
         console.output( "Unrecognized file: %s\n", fn );
         abort();
      }

      delete[] ext;
   }

   ide = new IDEController( this, hdFileName );

   // memory.cwrite(0x48F0, 0x55); // unlock backdoor

   uart.open( "uart.log", ios::out | ios::binary );
   uartIn = open( "uart.pipe", O_NONBLOCK );

   audio.open( "audio.log", ios::out | ios::binary );
      
   comments = new CommentMap( symbols );
   memory.findStrings(  );
   cpu.locatefunctions(  );
   cpu.go(  );
   cpu.setVersion( DEFAULT_VER );
}

Emulator::~Emulator(  ) {
   comments->dump( "gmemu.tag" );
   symbols->dump( "gmemu.map" );
   uart.close(  );

   delete ide;
   delete comments;
   delete symbols;
}


void Emulator::saveState(  ) {
  cpu.saveState();
  memory.saveState();
  io.saveState();
}

void Emulator::restoreState(  ) {
  cpu.restoreState();
  memory.restoreState();
  io.restoreState();
}
