/*  Project: Open Source Gmini Tools
    file: IDEController.h, header file for IDEController.cpp

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IDECONTROLLERH
#define IDECONTROLLERH

#include "types.h"
#include "trace.h"

class Emulator;

class IDEController {
 public:
   static const int REG_END = 0x100;
   IDEController( Emulator * gm, const char* hdFileName );
     ~IDEController(  );
   u16 readReg( u32 reg );
   void writeReg( u32 reg, u16 value );
   void transferData( int direction, u32 reg, u8 * memBuff, int numBytes );

 private:
      Emulator * gmini;
   void bufferNextSector(  );
   void flushSector(  );
   void readSector( int sector, u8 * buffer );
   void writeSector( int sector, u8 * buffer );

   u16 registers[REG_END];
   // only even values are used. (for programmer's 
   // convenience)
   int currentDirection;
   // direction of current transfer
   // 0: mem -> ide
   // 1: ide -> mem
   int nextSector;
   int pendingSectors;
   int pendingDataCount;
   u16 configData[0x100];
   u16 ideBuffer[0x100];

   bool hdWrite; // write to hd image enabled ?
   int hdfd;

};

#endif
