/*  Project: Open Source Gmini Tools
    file: LCDController.h, header file for LCDController.cpp

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LCDCONTROLLERH
#define LCDCONTROLLERH

#include "types.h"
#include "trace.h"

class Emulator;

class LCDController {
public:
   LCDController( Emulator * gm );
   ~LCDController(  );
   u8 read8( u32 reg );
   u16 read16( u32 reg );
   bool write8( u32 reg, u8 value );
   bool write16( u32 reg, u16 value );

   bool enabled();
   int bpp();
   int width();
   int height();
   int frameBuffer();   

private:
   u8 regs[0x40];

   u8 getReg8(int reg);
   u16 getReg16(int reg);
   u24 getReg24(int reg);
   void setReg8  (int reg, u8 value);
   void setReg16(int reg, u16 value);
   void setReg24(int reg, u24 value);

   /*
   u16 con0;
   u16 con1;
   u16 con2;
   u16 con3;
   u16 con4;
   u16 con5;
   u24 dma1_base;
   u24 dma2_base;
   u24 dma1_current;
   u24 dma2_current;
   u8  lsr;
   u16 red_high;
   u16 red_low;
   u16 green_high;
   u16 green_low;
   u16 blue;
   u8  dither_1_7;
   u8  dither_3_7;
   u8  dither_4_7;
   u8  dither_5_7;
   u8  dither_6_7;
   u8  dither_1_5;
   u8  dither_2_5;
   u8  dither_3_5;
   u8  dither_4_5;
   u8  dither_1_4;
   u8  dither_3_4;
   u8  dither_1_3;
   */
   
   enum lcdregs {
      LCNTR0  = 0x00, // LCD Control register 0 High               00H  R/W
      LCNTR1  = 0x02, // LCD Control register 1 High               00H  R/W
      LCNTR2  = 0x04, // LCD Control register 2 High               00H  R/W
      LCNTR3  = 0x06, // LCD Control register 3 High               00H  R/W
      LCNTR4  = 0x08, // LCD Control register 4 High               00H  R/W
      LCNTR5  = 0x0a, // LCD Control register 5 High               00H  R/W
      LDBAR1  = 0x0d, // LCD DMA Channel 1 Base address High       00H  R/W
      LDBAR2  = 0x11, // LCD DMA Channel 2 Base address High       00H  R/W
      LDCAR1  = 0x15, // LCD DMA Channel 1 Current address High    00H  R
      LDCAR2  = 0x19, // LCD DMA Channel 2 Current address High    00H  R
      LSR     = 0x1d, // LCD Status register                       00H  R/W
      LRLUTRH = 0x1e, // LCD Red Lookup Table High-High            00H  R/W
      LRLUTRL = 0x20, // LCD Red Lookup Table Low-High             00H  R/W
      LGLUTRH = 0x22, // LCD Green Lookup Table High-High          00H  R/W
      LGLUTRL = 0x24, // LCD Green Lookup Table Low-High           00H  R/W
      LBLUTRH = 0x26, // LCD Blue Lookup Table Low-High            00H  R/W
      LDP1_7  = 0x28, // LCD 1/7 Dither Pattern register           00H  R/W
      LDP3_7  = 0x29, // LCD 3/7 Dither Pattern register           00H  R/W
      LDP4_7  = 0x2a, // LCD 4/7 Dither Pattern register           00H  R/W
      LDP5_7  = 0x2b, // LCD 5/7 Dither Pattern register           00H  R/W
      LDP6_7  = 0x2c, // LCD 6/7 Dither Pattern register           00H  R/W
      LDP1_5  = 0x2e, // LCD 1/5 Dither Pattern register           00H  R/W
      LDP2_5  = 0x2f, // LCD 2/5 Dither Pattern register           00H  R/W
      LDP3_5  = 0x30, // LCD 3/5 Dither Pattern register           00H  R/W
      LDP4_5  = 0x31, // LCD 4/5 Dither Pattern register           00H  R/W
      LDP1_4  = 0x32, // LCD 1/4, 1/2 Dither Pattern register      00H  R/W
      LDP3_4  = 0x33, // LCD 3/4 Dither Pattern register           00H  R/W
      LDP1_3  = 0x34  // LCD 1/3, 2/3 Dither Pattern register      00H  R/W
   };
   Emulator * gmini;
};

#endif
