/*  Project: Open Source Gmini Tools
    file: MMIO.cpp, abstraction of memory mapped IO

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <fstream>
using namespace std;
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "types.h"
#include "Emulator.h"

const char
   MMIO::mapped[400][10] = {
   "PCON", "OSCCON", "WTCON", "IO_003", "BTCON", "BTCNT", "WDTEN", "WDTCON",
   "TACON", "TAPRE", "TADATAH", "TADATAL", "TACNTH", "TACNTL", "IO_00E",
   "IO_00F",
   "TBCON", "TBPRE", "TBDATAH", "TBDATAL", "TBCNTH", "TBCNTL", "IO_016",
   "IO_017",
   "TCCON", "TCPRE", "TCDATAH", "TCDATAL", "TCCNTH", "TCCNTL", "IO_01E",
   "IO_01F",
   "IRQ0H", "IRQ0L", "IMR0H", "IMR0L", "IPR0H", "IPR0L", "IIR0H", "IO_027",
   "IRQ1H", "IRQ1L", "IMR1H", "IMR1L", "IPR1H", "IPR1L", "IO_02E", "IO_02F",
   "P0", "P1", "P2", "P3", "P4", "P5", "P6", "P7",
   "P8", "P9", "P10", "IO_03B", "IO_03C", "IO_03D", "IO_03E", "IO_03F",
   "P0CON", "P1CON", "P2CONH", "P2CONL", "P3CONH", "P3CONL", "P3PUR", "IO_047",
   "P5CON", "P5PUR", "P5INTMODH", "P5INTMODL", "P5INTCON", "IO_04D", "IO_04E",
   "IO_04F",
   "P4CON", "P4INTCON", "P4INTMOD", "P6CON", "P7CON", "P8CON", "P9CON",
   "P10CON",
   "SMCON", "ECCNT", "ECCH", "ECCL", "ECCX", "ECCCLR", "ECCRSTH", "ECCRSTL",
   "PPDATA", "PPCDATA", "PPSCON", "PPSTAT", "PPCONH", "PPCONL", "PPINTCONH",
   "PPINTCONL",
   "PPINTPNDH", "PPINTPNDL", "PPACKD", "IO_06B", "IO_06C", "IO_06D", "IO_06E",
   "IO_06F",
   "SIOCON", "SIOPS", "SIODATA", "IO_073", "ADDATAH", "ADDATAL", "ADCON",
   "IO_077",
   "IO_078", "IO_079", "IO_07A", "IO_07B", "IO_07C", "IO_07D", "IO_07E",
   "IO_07F",
   "FUNADDR", "PWRMAN", "FRAMELO", "FRAMEHI", "INTREG", "INTENA", "EPINDEX",
   "IO_087",
   "IO_088", "EPDIR", "INCSR", "OUTCSR", "INMAXP", "OUTMAXP", "WRTCNTLO",
   "WRTCNTHI",
   "EP0FIFO", "EP1FIFO", "EP2FIFO", "EP3FIFO", "USB_0", "USB_1", "USB_2",
   "USB_3",
   "USB_4", "USB_5", "USB_6", "USB_7", "USB_8", "USB_9", "USB_A", "USB_B",
   "IISCON0", "IISMODE0", "IISPTR0", "IO_0A3", "IISCON1", "IISMODE1",
   "IISPTR1",
   "IO_0A7",
   "PLL0DATAH", "PLL0DATAL", "PLL0CON", "IO_0AB", "PLL1DATAH", "PLL1DATAL",
   "PLL1CON", "IO_0AF",
   "LCON", "UCON", "USSR", "TBR", "RBR", "UBRDR", "UPEND", "IO_0B7",
   "IICCON", "IICSR", "IICDATA", "IICADDR", "IICPS", "IICCNT", "IO_0BE",
   "IO_0BF",
   "BUF64", "BUF64_01", "BUF64_02", "BUF64_03", "BUF64_04", "BUF64_05",
   "BUF64_06", "BUF64_07",
   "BUF64_08", "BUF64_09", "BUF64_0A", "BUF64_0B", "BUF64_0C", "BUF64_0D",
   "BUF64_0E", "BUF64_0F",
   "BUF64_10", "BUF64_11", "BUF64_12", "BUF64_13", "BUF64_14", "BUF64_15",
   "BUF64_16", "BUF64_17",
   "BUF64_18", "BUF64_19", "BUF64_1A", "BUF64_1B", "BUF64_1C", "BUF64_1D",
   "BUF64_1E", "BUF64_1F",
   "BUF64_20", "BUF64_21", "BUF64_22", "BUF64_23", "BUF64_24", "BUF64_25",
   "BUF64_26", "BUF64_27",
   "BUF64_28", "BUF64_29", "BUF64_2A", "BUF64_2B", "BUF64_2C", "BUF64_2D",
   "BUF64_2E", "BUF64_2F",
   "BUF64_30", "BUF64_31", "BUF64_32", "BUF64_33", "BUF64_34", "BUF64_35",
   "BUF64_36", "BUF64_37",
   "BUF64_38", "BUF64_39", "BUF64_3A", "BUF64_3B", "BUF64_3C", "BUF64_3D",
   "BUF64_3E", "BUF64_3F",
   "IDSC0H", "IDSC0L", "IDSC1H", "IDSC1L", "IDSC2H", "IDSC2L", "IDSC3H",
   "IDSC3L",
   "RANCON", "LFSR8", "LFSR16H", "LFSR16L", "INTMODE", "FMCON", "FMCLK",
   "FMDAT",
   // 110h
   "MIUSCFG", "IO_111", "MIUDCMD", "MIUDCFG", "MIUDCNTH", "MIUDCNTL",
   "IO_116", "IO_117",
   "IO_118", "IO_119", "IO_11A", "IO_11B", "IO_11C", "IO_11D", "IO_11E",
   "IO_11F",
   "DDMACOM", "DDMACFG", "IO_122", "DDMAIADRX", "DDMAIADRH", "DDMAIADRL",
   "IO_126", "DDMAEADRX",
   "DDMAEADRH", "DDMAEADRL", "DDMANUMH", "DDMANUML", "DDMACNTH", "DDMACNTL",
   "DDMANUMX", "DDMACNTX",
   "YDMACOM", "YDMACFG", "YDMAIADRH", "YDMAIADRL", "IO_134", "YDMAEADRX",
   "YDMAEADRH", "YDMAEADRL",
   "YDMANUMH", "YDMANUML", "YDMACNTH", "YDMACNTL", "IO_13C", "IO_13D",
   "IO_13E",
   "IO_13F",
   // 140h
   "CACHECON", "ICBASEH", "ICBASEL", "IO_143", "DCBASEH", "DCBASEL",
   "IO_146", "IO_147",
   "IO_148", "IO_149", "IO_14A", "IO_14B", "IO_14C", "IO_14D", "IO_14E",
   "IO_14F",
   "LCNTR0H", "LCNTR0L", "LCNTR1H", "LCNTR1L", "LCNTR2H", "LCNTR2L", "LCNTR3H",
   "LCNTR3L",
   "LCNTR4H", "LCNTR4L", "LCNTR5H", "LCNTR5L", "IO_15C", "LDBAR1H", "LDBAR1M",
   "LDBAR1L",
   "IO_160", "LDBAR2H", "LDBAR2M", "LDBAR2L", "IO_164", "LDCAR1H", "LDCAR1M",
   "LDCAR1L",
   "IO_168", "LDCAR2H", "LDCAR2M", "LDCAR2L", "IO_16C", "LSR", "LRLUTRHH",
   "LRLUTRHL",
   "LRLUTRLH", "LRLUTRLL", "LGLUTRHH", "LGLUTRHL", "LGLUTRLH", "LGLUTRLL",
   "LBLUTRH", "LBLUTRL",
   "LDP1_7", "LDP3_7", "LDP4_7", "LDP5_7", "LDP6_7", "IO_17D", "LDP1_5",
   "LDP2_5",
   "LDP3_5", "LDP4_5", "LDP1_4", "LDP3_4", "LDP1_3", "IO_185", "IO_186",
   "IO_187",
   "IO_188", "IO_189", "IO_18A", "IO_18B", "IO_18C", "IO_18D", "IO_18E",
   "IO_18F"
};

MMIO::MMIO( Emulator * gm ) {
   gmini = gm;

   for( int i = 0; i < MMIO_END; i++ )
      io[i] = 0;

   // io[P5] |= 0x10; // USB plug in

   io[P7] |= 0x80;              // unplug DC (else we see battery guy)
   io[P7] |= 0x0C;              /* When bits[32]= 0, CFC is detected */
   io[P7] |= 0x02;              /* battery always high */

   flashState = FS_1;
}

u8 MMIO::read8( u22 address ) {
#ifdef __GMEMU_DEBUG__
   ++ioaccess[address][0];
#endif
   if( address >= 0x150 )
      return gmini->lcdi.read8( address - 0x150 );
   u8 value = io[address];
   switch ( address ) {
      case PLL0CON:
      case PLL1CON:
         value |= 0x02;         /* PLL output freq. instantly stabilizes to target freq. */
         break;
      case YDMACOM:
         value &= 0xfc;         /* Always idle & complete: dma is infinitely fast. */
         break;
      case DDMACOM:
         value &= 0xfc;         /* Always idle: dma is infinitely fast. */
         break;
      case USSR:
         value |= 0x40;         // not sure what this does
         break;
   }
   if( TRACE_IO ||
       (TRACE_I2S && (address>=IISCON0 && address <= IISPTR1)))
      gmini->tracer.log( "IO read   8: %10s ->   0x%02x\n", mapped[address], value );
#ifdef __GMEMU_DEBUG__
   console.output( "Read from  (u8)io[%s] = 0x%x\n", mapped[address], value );
#endif

   return value;
}

u16 MMIO::read16( u22 address ) {
#ifdef __GMEMU_DEBUG__
   ++ioaccess[address][0];
   ++ioaccess[address + 1][0];
#endif
   if( address >= 0x150 )
      return gmini->lcdi.read16( address - 0x150 );
   u16 value = ( u16 ) ( ( io[address] << 8 ) | io[address + 1] );
   if( TRACE_IO )
      gmini->tracer.log( "IO read  16: %10s -> 0x%04x\n", mapped[address], value );
#ifdef __GMEMU_DEBUG__
   console.output( "Read from (u16)io[%s] = 0x%x\n", mapped[address], value );
#endif
   return value;
}

bool MMIO::write8( u22 address, u8 value ) {
#ifdef __GMEMU_DEBUG__
   ++ioaccess[address][1];
#endif
   if( TRACE_IO || 
       (TRACE_I2S && (address>=IISCON0 && address <= IISPTR1)))
      gmini->tracer.log( "IO write  8: %10s <-   0x%02x\n", mapped[address], value );
#ifdef __GMEMU_DEBUG__
   console.output( "Write to io[%s] =  (u8)0x%x\n", mapped[address], value );
#endif
   if( address >= 0x150 )
      return gmini->lcdi.write8( address - 0x150, value );
   switch ( address ) {
      case DDMACOM:
         switch ( ( value & 0x0C ) >> 2 ) {
            case 0:
               if( TRACE_DMA )
                  gmini->tracer.log( "DDMA stop" );
               assert( false );
               break;
            case 1:
               // DDMA transfer start
               ddmaStart(  );
               break;
            case 2:
               // DDMA transfer continue
               assert( false );
               break;
         }
         break;
      case YDMACOM:
         switch ( ( value & 0x30 ) >> 4 ) {
            case 0:
	      //YDMA transfer stop
               assert( false );
               break;
            case 1:
               // YDMA transfer start
               ydmaStart(  );
               break;
            case 2:
               // YDMA transfer continue
               assert( false );
               break;
         }
         break;
      case P0:
         gmini->lcd.write( io[P2], value );
         break;
      case P3:
      case P3CONH:
         io[address] = value;
         p3Interact( address == P3CONH );
         break;
      case TBR:
	gmini->uart << ( char )( value & 0xFF ) << flush;
         break;
      case ADCON:
         u16 data;
         gmini->adc.interact( value, data );
         io[ADCON] = value;
         rawwrite16( ADDATAH, data );
         break;
      default:
         io[address] = value;
         break;
   }
   return true;
}

void MMIO::p3Interact( bool readLines ) {
   bool sda = io[P3] & 0x20;
   bool scl = io[P3] & 0x10;
   bool sdaModeWrite = ( io[P3CONH] & 0x0C ) != 0;
   bool sclModeWrite = ( io[P3CONH] & 0x03 ) != 0;
   static char arrows[2][3] = {"<-","->"};
   if (TRACE_I2C)
     gmini->tracer.log("I2C master: SCL %s %d  SDA %s %d",
   		       arrows[sclModeWrite], scl,
   		       arrows[sdaModeWrite], sda);
   
   gmini->i2c.lineInteractMaster( I2CBus::SCL, sclModeWrite, scl );
   gmini->i2c.lineInteractMaster( I2CBus::SDA, sdaModeWrite, sda );

   if (readLines) {
     // Sometimes archos firmware does things in this order:
     // 1. Set line value, 2. set output mode.

     // Thus when P3 is written we don't update its value
     // with the bus value even in read mode.
     
     if( !sclModeWrite )
       io[P3] = ( io[P3] & 0xEF ) | ( scl ? 0x10 : 0 );
     if( !sdaModeWrite )
       io[P3] = ( io[P3] & 0xDF ) | ( sda ? 0x20 : 0 );
   }
   if (TRACE_I2C)
     gmini->tracer.log("I2C master: SCL %s %d  SDA %s %d",
   		       arrows[sclModeWrite], scl,
   		       arrows[sdaModeWrite], sda);

}

#define assertFlash( x ) do {				\
  if (!(x)) gmini->tracer.log("FLASH protocol error (%d)", __LINE__);	\
} while (0)

void MMIO::ddmaStart(  ) {
   // see datasheet p. 454
   u8 cfg = io[DDMACFG];
   int interruptEnable = cfg & 1;
   // inform completion by interrupt
   assert( interruptEnable == 0 );  // not implemented
   int direction = ( cfg & 0x02 ) >> 1;
   // 0: external to internal (aka read)
   // 1: internal to external (aka write)
   int internalMem = ( cfg & 0x0C ) >> 2;
   // 0: Program memory
   // 1: X-Memory
   // >1: Data cache memory
   // i'm not sure it does what the datasheet says anyways
   u32 internalAddress =
     ( ( ( u32 ) io[DDMAIADRX] << 16 ) | ( ( u32 ) io[DDMAIADRH] << 8 ) | ( u32 ) io[DDMAIADRL] ) >> 1;
   // lsb is ignored 
   u32 externalAddress = ( ( ( u32 ) io[DDMAEADRX] << 16 ) | ( ( u32 ) io[DDMAEADRH] << 8 ) | ( u32 ) io[DDMAEADRL] );
   u32 number = ( ( u32 ) io[DDMANUMX] << 16 ) | ( ( u32 ) io[DDMANUMH] << 8 ) | ( ( u32 ) io[DDMANUML] );

   // convert in bytes (from words)
   internalAddress = internalAddress << 1;
   externalAddress = externalAddress << 1;
   number = number << 1;

   // report what we should do
   if( TRACE_DMA )
      gmini->tracer.log( "DDMA dir=%d mem=%d int=%x ext=%x num=%x",
                         direction, internalMem, internalAddress, externalAddress, number );

   // the following is just guess.
   if( internalMem == 1 ) {
      u22 intAddr = internalAddress + 0x200000;
      if( ( externalAddress & 0x800000 ) != 0 ) {
	assert((externalAddress & 0xFFFF) == 0);
         u32 ideReg = externalAddress >> 16;
         if( number == 2 ) {
            if( direction == 0 ) {
               gmini->memory.dwrite16( intAddr, ( gmini->ide->readReg( ideReg ) ) );
            }
            else {
               gmini->ide->writeReg( ideReg, ( gmini->memory.dread16( intAddr ) ) );
            }
         } else {
            gmini->ide->transferData( direction, ideReg, 
				      gmini->memory.getdmem(  ) + intAddr, number );
         }
      } else {
	// flash chip access
	assert( number == 2 );
	int flashAddr = externalAddress >> 1; // 16 bits words 
	if (direction == 1) {
	  u16 data = gmini->memory.dread16( intAddr );
	  if (TRACE_FLASH)
	    gmini->tracer.log("FLASH write: addr=%04x data=%04x", flashAddr, data);
	  switch(flashState) {
	  case FS_1:
	  case FS_ERASE1:
	    assertFlash(flashAddr == 0x5555);
	    assertFlash(data = 0xAA);
	    flashState = (flashState == FS_1) ?
	      FS_2 : FS_ERASE2;
	    break;
	  case FS_2:
	  case FS_ERASE2:
	    assertFlash(flashAddr == 0x2AAA);
	    assertFlash(data = 0x55);
	    flashState = (flashState == FS_2) ?
	      FS_COMMAND : FS_ERASE_COMMAND;
	    break;
	  case FS_COMMAND:
	    assertFlash(flashAddr == 0x5555);
	    switch(data) {
	    case 0x80:
	      flashState = FS_ERASE1;
	      break;	      
	    case 0x90:
	      flashState = FS_SOFTWARE_ID;
	      break;
	    case 0xA0:
	      flashState = FS_WORD_PROGRAM;
	      break;
	    case 0xF0:
	      flashState = FS_1;
	      break;
	    default:
	      assertFlash(false);
	    }
	    break;
	  case FS_ERASE_COMMAND:
	    switch(data) {
	    case 0x30:
	      if (TRACE_FLASH)
		console.output("FLASH erase sector at %x\n", flashAddr*2);
	      for (int i=0; i < 0x2048; i++)
		gmini->memory.cwrite16((flashAddr+i)*2, 0xFFFF);
	      flashState = FS_1;
	      break;
	    default:
	      console.output("Unhandled erase command: addr=%04x data=%02x\n",
			     flashAddr, data);     
	      flashState = FS_1;
	      assertFlash(false);
	    }
	    break;
	  case FS_WORD_PROGRAM:
	    gmini->memory.cwrite16(externalAddress, data);
	    flashState = FS_1;
	    break;
	  case FS_SOFTWARE_ID:
	    switch(flashAddr) {
	    case 0xF0:
	      flashState = FS_1;
	      break;
	    case 0x5555:
	      assertFlash(data == 0xAA);
	      flashState = FS_2; // hack
	      break;
	    default:
	      assertFlash(false);
	    }
	    break;
	  }
	  // console.output("FLASH state: %d\n", flashState);	  
	} else {
	  // console.output("FLASH rd0 : addr=%0x\n", flashAddr);
	  u16 data=0xABCD;
	  switch(flashState) {
	  case FS_1:
	    data = gmini->memory.cread16(externalAddress);
	    break;
	  case FS_SOFTWARE_ID:
	    switch(flashAddr) {
	    case 0: 
	      data = 0xBF; // SST id
	      break;
	    case 1:
	      // data = 0x273F; // SST 39WF 800A
	      // data = 0x225B;
	      data = 0x2781; //  SST 39VF 800A
	      break;
	    default:
	      assertFlash(false);
	    }
	    break;
	  default:
	    assertFlash(false);
	  }
	  if (TRACE_FLASH)
	    gmini->tracer.log("FLASH read: addr=%04x data=%04x", flashAddr, data);
	  //console.output("FLASH read: addr=%0x data=%x\n", flashAddr, data);
	  gmini->memory.dwrite16( intAddr, data );
	}
	
      } 
   }
   else {
      assert( direction == 0 );
      gmini->memory.cmemcopy( 0x3FC000 | internalAddress, externalAddress, number );
   }

}

void MMIO::ydmaStart(  ) {
   // see datasheet p. 443
   int block = (io[YDMACOM] & 0x40) >> 6;
   u8 cfg = io[YDMACFG];
   int interruptEnable = cfg & 1;
   // inform completion by interrupt
   assert( interruptEnable == 0 );  // not implemented
   int direction = ( cfg & 0x02 ) >> 1;
   // 0: external to internal (aka read)
   // 1: internal to external (aka write)
   u32 internalAddress = rawread16(YDMAIADRH);
   // lsb is ignored 
   u32 externalAddress = ( ( ( u32 ) io[YDMAEADRX] << 16 ) | ( ( u32 ) io[YDMAEADRH] << 8 ) | ( u32 ) io[YDMAEADRL] );
   u32 number =  ( ( u32 ) io[YDMANUMH] << 8 ) | ( ( u32 ) io[YDMANUML] );


   // report what we  do
   if (TRACE_DMA)
     gmini->tracer.log( "YDMA dir=%d blk=%d int=%x ext=%x num=%x",
			direction, block, internalAddress, externalAddress, number );

   gmini->memory.ydma(direction, block, internalAddress, externalAddress, number);
}

bool MMIO::write16( u22 address, u16 value ) {
#ifdef __GMEMU_DEBUG__
   ++ioaccess[address][1];
   ++ioaccess[address + 1][1];
#endif
   if( TRACE_IO ||
       (TRACE_I2S && (address >= IISBUF && address <= IISBUF_LAST)))
      gmini->tracer.log( "IO write 16: %10s <- 0x%04x\n", mapped[address], value );
#ifdef __GMEMU_DEBUG__
   console.output( "Write to io[%s] = (u16)0x%x\n", mapped[address], value );
#endif
   if( address >= 0x150 )
      return gmini->lcdi.write16( address - 0x150, value );
   io[address] = ( u8 ) ( ( value & 0xff00 ) >> 8 );
   io[address + 1] = value & 0xff;
   return true;
}

void MMIO::dump(  ) {
#ifdef __GMEMU_DEBUG__
   for( u32 i = 0; i < MMIO_END; ++i ) {
      if( ( ioaccess[i][0] != 0 ) || ( ioaccess[i][1] != 0 ) )
         printf( "%03x io[%s]=0x%02x R:%d W:%d\n", i, mapped[i], io[i], ioaccess[i][0], ioaccess[i][1] );
   }
#endif
}

bool MMIO::dump8( const char *datafile, u8 byte ) {
//    ofstream data( datafile, ios::out | ios::binary | ios::app );
//    if( !data ) {
//       console.output( "Could not open port file: %s\n", datafile );
//       return false;
//    }
//    data.write( ( char * )( &byte ), 1 );
//    data.close(  );
   return true;
}

bool MMIO::dump16( const char *datafile, u16 byte ) {
//    ofstream data( datafile, ios::out | ios::binary | ios::app );
//    if( !data ) {
//       console.output( "Could not open port file: %s\n", datafile );
//       return false;
//    }
//    data.write( ( ( char * )( &byte ) ) + 1, 1 );
//    data.write( ( char * )( &byte ), 1 );
//    data.close(  );
   return true;
}

u8 MMIO::rawread8( u22 address ) {
   return io[address];
}

u16 MMIO::rawread16( u22 address ) {
   u16 value = ( u16 ) ( ( io[address] << 8 ) | io[address + 1] );
   return value;
}

void MMIO::rawwrite8( u22 address, u8 value ) {
   io[address] = value;
}

void MMIO::rawwrite16( u22 address, u16 value ) {
   io[address] = ( u8 ) ( ( value & 0xff00 ) >> 8 );
   io[address + 1] = value & 0xff;
}

static const char* const datafile= "stateMMIO.bin"; 

void MMIO::saveState() {
  ofstream data;
  data.open( datafile, ios::out | ios::binary );
   if( !data ) {
      console.output( "Could not open MMIO file: %s\n", datafile );
      return;
   }
   data.write( ( char * )io, MMIO_END );
   data.close(  );
}

void MMIO::restoreState() {
  ifstream data;
  data.open( datafile, ios::in | ios::binary );
   if( !data ) {
      console.output( "Could not open MMIO file: %s\n", datafile );
      return;
   }
   u32 i = 0;
   u8 ch;
   while( !( data.eof(  ) ) && i < MMIO_END ) {
      ch = data.get(  );
      io[i++] = ch;
   }
   data.close(  );
   console.output( "Loaded %d bytes to IO registers\n", i );
}
