%{
#include <iostream>
#include <stdio.h>
#include <ctype.h>
using namespace std;
#include "types.h"
#include "Emulator.h"
#define YYDEBUG 0

extern "C" {
 int yyparse();
 int yylex();
 char* yytext;
 int yy_set_input(char* newInput);
}

static int yyerror(char *s);

#define PROMPT "gmemu> "

 extern bool cmdexit;
 static Emulator* gmini_loc;

void display_help() {
//                  12345678901234567890123456789012345678901234567890123456789012345678901234567890
   console.output( "gmemu commands:\n" );
   console.output( "  help              -- displays this message\n" );
   console.output( "  quit              -- quit gmemu\n" );
   console.output( "execution:\n" );
   console.output( "  go                -- force PC back to start address\n" );
   console.output( "  go [address]      -- force PC to address specified\n" );
   console.output( "  step              -- step\n" );
   console.output( "  walk              -- walk\n" );
   console.output( "  walk [address]    -- walk until address specified\n" );
   console.output( "  jog               -- jog\n" );
   console.output( "  jog [address]     -- jog until address specified\n" );
   console.output( "  run               -- run\n" );
   console.output( "  run [address]     -- run until address specified\n" );
   console.output( "breakpoints:\n" );
   console.output( "  bt                -- backtrace\n" );
   console.output( "  bpx [address]     -- sets a breakpoint at execution of the specified address\n" );
   console.output( "  bpm [address]     -- sets a breakpoint at memory access of specified address\n" );
   console.output( "  bpr [start] [end] -- sets a breakpoint for memory access within a range\n" );
   console.output( "  list              -- display a list of set breakpoints\n" );
   console.output( "  delete [number]   -- delete the specified breakpoint\n" );
   console.output( "  clear             -- delete all breakpoints\n" );
   console.output( "status display:\n" );
   console.output( "  lcd               -- dump lcd display\n" );
   console.output( "  registers         -- dump registers\n" );
   console.output( "  ops               -- dump ops\n" );
   console.output( "  ops [number]      -- dump specified number of ops\n" );
   console.output( "  mmio              -- dump io statistics\n" );
   console.output( "  dump [address]    -- displays contents of memory at specified address\n" );
   console.output( "  load [filename]   -- import a file containing memory values\n" );
   console.output( "  lcd               -- dump lcd display\n" );
   console.output( "file writing:\n" );
   console.output( "  ram               -- dumps contents of memory to a file named ram.bin\n" );
   console.output( "  flash             -- dumps contents of flash to a file named flash.bin\n" );
   console.output( "state data:\n" );
   console.output( "  save              -- saves current state data to files\n" );
   console.output( "  restore           -- restores state data from files\n" );
   console.output( "modify state: \n" );
   console.output( "  set rX value\n" );
}; 

%}

%union VALTYPE {
	unsigned int address;
	unsigned int number;
	int integer;
	char *string;
}

%token _BPM _BPR _BPX _BT _CLEAR _DELETE _DISASM _FLASH _GO _GOTO _HELP
%token _JOG _LIST _DUMP _LCD
%token _MMIO _NEXT _OPS _QUIT _RAM _REGS _RESTORE _RUN _RUN2RET _SAVE _SET _STEP
%token _WALK _ANYOTHER
%token _LOAD _EOL _TRACE
%token <address> _ADDRESS
%token <number> _NUMBER
%token _R _W _X
%token <integer> _INTEGER
%token <string> _FILENAME
%token <string> _IDENTIFIER
%token <number> _RREG

%type <address> Symbol

%start Input

%%
Input:
	| Command
	;

Line:	  _EOL 
	| Command _EOL
	;

Symbol:	  _ADDRESS                      { $$ = $1; }
	| _IDENTIFIER                   { $$ = gmini_loc->symbols->resolveInteractive($1); }
	;

Command:
	  _BPM Symbol Access		{ gmini_loc->BL.BPM($2, $<integer>3);
					  return 0; }
	| _BPR _ADDRESS _ADDRESS Access	{ gmini_loc->BL.BPR($<address>2, $<address>3, $<integer>4);
					  return 0; }
	| _BPX Symbol			{ gmini_loc->BL.BPX($2);
					  return 0; }
        | _BT                           { gmini_loc->cpu.dumpBacktrace(cout);
	                                  return 0; }
	| _CLEAR			{ gmini_loc->BL.clear();
					  return 0; }
	| _DELETE _NUMBER		{ gmini_loc->BL.del($<number>2);
					  return 0; }
	| _DISASM Symbol		{ gmini_loc->cpu.disasm($2);
					  return 0; }
	| _FLASH			{ gmini_loc->memory.dumpmem( "flash.bin" );
					  return 0; }
	| _GO Symbol			{ gmini_loc->cpu.go($2);
					  return 0; }
	| _GO				{ gmini_loc->cpu.go();
					  return 0; }
	| _HELP				{ display_help();
					  return 0; }
	| _JOG _ADDRESS			{ gmini_loc->cpu.jog($<address>2);
					  return 0; }
	| _JOG				{ gmini_loc->cpu.jog();
					  return 0; }
        | _LCD                          { gmini_loc->lcd.dump(); 
                                          return 0; }
	| _LIST				{ gmini_loc->BL.Print();
					  return 0; }
	| _MMIO				{ gmini_loc->io.dump();
					  return 0; }
        | _NEXT                         { gmini_loc->cpu.stepOver(); 
 	                                  return 0; }
        | _OPS				{ gmini_loc->cpu.dumpops( );
					  return 0; }
	| _QUIT				{ console.output( "exiting...\n" );
					  cmdexit = true;
					  return 0; }
	| _RAM				{ gmini_loc->memory.dumpram( "ram.bin" );
					  return 0; }
	| _REGS				{ gmini_loc->cpu.dumpregs();
					  return 0; }
	| _RESTORE			{ gmini_loc->restoreState();
					  return 0; }
	| _RUN Symbol			{ gmini_loc->cpu.run($2);
					  return 0; }
	| _RUN				{ gmini_loc->cpu.run();
					  return 0; }
	| _RUN2RET			{ gmini_loc->cpu.run2ret();
					  return 0; }
	| _SAVE				{ gmini_loc->saveState();
					  return 0; }
   | _SET _RREG _NUMBER { gmini_loc->cpu.setR($2, $3);
            return 0;}
	| _STEP				{ gmini_loc->cpu.step(); gmini_loc->cpu.dumpops(); 					  return 0; }

	| _TRACE			{ gmini_loc->tracer.enabled = 
	 				  !gmini_loc->tracer.enabled;
					  return 0;
					}
	| _WALK _ADDRESS		{ gmini_loc->cpu.walk($<address>2);
					  return 0; }
	| _WALK				{ gmini_loc->cpu.walk();
					  return 0; }
	| _DUMP Symbol			{ gmini_loc->memory.dumpScreen($2);
					  return 0; }
	| _DUMP Symbol _ADDRESS	{ gmini_loc->memory.dumpScreen($<address>2, $3);
					  return 0; }
	| _LOAD _FILENAME		{ gmini_loc->memory.initialize($<string>2);
					  free($<string>2);
					  return 0; }
	;
Access:
	  { $<integer>$ = 0; }
	| Access _R { $<integer>$ = $<integer>1 | Breakpoint::R; }
	| Access _W { $<integer>$ = $<integer>1 | Breakpoint::W; }
	| Access _X { $<integer>$ = $<integer>1 | Breakpoint::X; }
 	;
	
%%

int yyerror(char *s) {
	cerr << "Syntax error near " << yytext << endl;
	return -1;
}

int parsecmd(Emulator* gm) {
	gmini_loc = gm;
	console.output( PROMPT );
	static char cmd[2048]; 
	char buffer[2048];
	cin.getline(buffer, 2048);
	//cout << "#cmd = " << strlen(buffer) << endl;
	if (buffer[0])
	   strcpy(cmd, buffer);
	yy_set_input(cmd);
	int res = yyparse();
	return res;
}

void startparser() {
        //yydebug = 1;
	cmdexit = false;
}

void stopparser() {
}

