
	.section .text
	.global	_start
_start:
	;; disable all interrupts
 	clrsr fe
	clrsr ie
	clrsr te

	;; copy data section from flash to ram.

	ld a11, #(_datacopy)	; where the data section is in the flash
	ld a8, #(_datastart)	; destination

	ld a9, #_datasize
	ld r6, e9
	cmp eq, r6, #0
	brf .data_copy_loop
	cmp eq, r9, #0
	brt .data_copy_end
.data_copy_loop:	
	ldc r2, @a11
	ldw @[a8 + 0], r2
	add a11, #0x2
	add a8, #0x2
	sub r9, #0x2
	sbc r6, #0
	cmp ugt, r6, #0
	brt .data_copy_loop
	cmp ugt, r9, #0
	brt .data_copy_loop
.data_copy_end:

	ld r2, #0
	ld a8, #(_bssstart)	; destination
	ld a9, #_bsssize
	
	ld r6, e9
	cmp eq, r6, #0
	brf .bss_init_loop
	cmp eq, r9, #0
	brt .bss_init_end
.bss_init_loop:	
	ldw @[a8 + 0], r2
	add a8, #0x2
	sub r9, #0x2
	sbc r6, #0
	cmp ugt, r6, #0
	brt .bss_init_loop
	cmp ugt, r9, #0
	brt .bss_init_loop
.bss_init_end:

	;; enable UART TX on P2.6
	ld a8, #0x3F0000
	ldb r0, @[a8 + 0x42]
	or r0, 4
	ldb @[a8 + 0x42], r0
	
	
	;; FIXME: initialize stack.
	;; stack lies between end of data and 07FFFE
	ld a15, #0x07fffe

	;; rest of the setup in C
	jsr _cpu_setup

	;; enable IRQs (keep traps and fast interrupts low)
	setsr ie

	
	;; go!
	jsr _main
	
	;; reset
	ld a10, #0
	ldc r10, @a10
	jmp a10

	
	.section .vectors, "ax"
irq_handler:

PUSH R0, R1
PUSH R2, R3
PUSH R4, R5
PUSH R6, R7
PUSH A8, A9
PUSH A10, A11
PUSH A12, A13
PUSH A14
LD A13, #0x3F0000
LDB R0, @[A13 + 0x26]
ADD R0, R0
LD A10, #_interruptVector
LDW A13, @[A10 + R0]
JSR A13
POP A14
POP A13, A12
POP A11, A10
POP A9, A8
POP R7, R6
POP R5, R4
POP R3, R2
POP R1, R0
RET_IRQ
	
